@@ -18,8 +18,8 @@ module Text.XML.HaXml.DtdToHaskell.TypeDef
   , name, name_, name_a, name_ac, name_f, mangle, manglef
   ) where
 
-import Char (isLower, isUpper, toLower, toUpper, isDigit)
-import List (intersperse)
+import Data.Char (isLower, isUpper, toLower, toUpper, isDigit)
+import Data.List (intersperse)
 import Text.PrettyPrint.HughesPJ
 
 
