@@ -155,13 +155,6 @@ thd3 (_,_,a) = a
 
 ---- Auxiliary Parsing Functions ----
 
--- | Parse a bracketed item, discarding the brackets AND NOT using adjustErrBad
-myBracket :: PolyParse p => p bra -> p ket -> p a -> p a
-myBracket open close p = do
-    do { open    `adjustErr` ("Missing opening bracket:\n\t"++)
-       ; p `discard` (close `adjustErr` ("Missing closing bracket:\n\t"++))
-       }
-
 -- | XParser is just a specialisation of the PolyState parser.
 type XParser a = Parser SymTabs (Posn,TokenT) a
 
@@ -307,7 +300,7 @@ document = do
 -- | Return an XML comment.
 comment :: XParser Comment
 comment = do
-    myBracket (tok TokCommentOpen) (tok TokCommentClose) freetext
+    bracket (tok TokCommentOpen) (tok TokCommentClose) freetext
 --  tok TokCommentOpen
 --  commit $ do
 --    c <- freetext
@@ -327,7 +320,7 @@ processinginstruction = do
 cdsect :: XParser CDSect
 cdsect = do
     tok TokSectionOpen
-    bracket (tok (TokSection CDATAx)) (tok TokSectionClose) chardata
+    bracket (tok (TokSection CDATAx)) (commit $ tok TokSectionClose) chardata
 
 prolog :: XParser Prolog
 prolog = do
@@ -358,7 +351,7 @@ versioninfo :: XParser VersionInfo
 versioninfo = do
     (word "version" `onFail` word "VERSION")
     tok TokEqual
-    bracket (tok TokQuote) (tok TokQuote) freetext
+    bracket (tok TokQuote) (commit $ tok TokQuote) freetext
 
 misc :: XParser Misc
 misc =
@@ -374,7 +367,7 @@ doctypedecl = do
     commit $ do
       n   <- qname
       eid <- maybe externalid
-      es  <- maybe (bracket (tok TokSqOpen) (tok TokSqClose)
+      es  <- maybe (bracket (tok TokSqOpen) (commit $ tok TokSqClose)
                             (many (peRef markupdecl)))
       blank (tok TokAnyClose)  `onFail` failP "missing > in DOCTYPE decl"
       return (DTD n eid (case es of { Nothing -> []; Just e -> e }))
@@ -408,7 +401,7 @@ sddecl = do
     (word "standalone" `onFail` word "STANDALONE")
     commit $ do
       tok TokEqual `onFail` failP "missing = in 'standalone' decl"
-      bracket (tok TokQuote) (tok TokQuote)
+      bracket (tok TokQuote) (commit $ tok TokQuote)
               ( (word "yes" >> return True) `onFail`
                 (word "no" >> return False) `onFail`
                 failP "'standalone' decl requires 'yes' or 'no' value" )
@@ -425,7 +418,7 @@ element = do
              ,  do tok TokAnyClose
                    cs <- many content
                    p  <- posn
-                   m  <- bracket (tok TokEndOpen) (tok TokAnyClose) qname
+                   m  <- bracket (tok TokEndOpen) (commit $ tok TokAnyClose) qname
                    checkmatch p n m
                    return (Elem n as cs))
            ] `adjustErr` (("in element tag "++n++",\n")++)
@@ -445,7 +438,7 @@ element = do
                  manyFinally content
                              (do p <- posn
                                  m <- bracket (tok TokEndOpen)
-                                              (tok TokAnyClose) qname
+                                              (commit $ tok TokAnyClose) qname
                                  checkmatch p n m)
       ) `adjustErrBad` (("in element tag "++printableName n++",\n")++)
 
@@ -515,7 +508,7 @@ elementdecl = do
     blank (tok TokAnyClose) `onFail` failBadP
        ("expected > terminating ELEMENT decl"
        ++"\n    element name was "++show (printableName n)
-       ++"\n    contentspec was "++(\ (ContentSpec p)-> show p) c)
+       ++"\n    contentspec was "++(\ (ContentSpec p)-> debugShowCP p) c)
     return (ElementDecl n c)
 
 contentspec :: XParser ContentSpec
@@ -530,29 +523,29 @@ contentspec =
 
 choice :: XParser [CP]
 choice = do
-    myBracket (tok TokBraOpen `debug` "Trying choice")
-              (blank (tok TokBraClose `debug` "Succeeded with choice"))
-              (peRef cp `sepBy1` blank (tok TokPipe))
+    bracket (tok TokBraOpen `debug` "Trying choice")
+            (blank (tok TokBraClose `debug` "Succeeded with choice"))
+            (peRef cp `sepBy1` blank (tok TokPipe))
 
 sequence :: XParser [CP]
 sequence = do
- myBracket (tok TokBraOpen `debug` "Trying sequence")
-           (blank (tok TokBraClose `debug` "Succeeded with sequence"))
-           (peRef cp `sepBy1` blank (tok TokComma))
+    bracket (tok TokBraOpen `debug` "Trying sequence")
+            (blank (tok TokBraClose `debug` "Succeeded with sequence"))
+            (peRef cp `sepBy1` blank (tok TokComma))
 
 cp :: XParser CP
 cp = oneOf [ ( do n <- qname
                   m <- modifier
                   let c = TagName n m
-                  return c `debug` ("ContentSpec: name "++show c))
+                  return c `debug` ("ContentSpec: name "++debugShowCP c))
            , ( do ss <- sequence
                   m <- modifier
                   let c = Seq ss m
-                  return c `debug` ("ContentSpec: sequence "++show c))
+                  return c `debug` ("ContentSpec: sequence "++debugShowCP c))
            , ( do cs <- choice
                   m <- modifier
                   let c = Choice cs m
-                  return c `debug` ("ContentSpec: choice "++show c))
+                  return c `debug` ("ContentSpec: choice "++debugShowCP c))
            ] `adjustErr` (++"\nwhen looking for a content particle")
 
 modifier :: XParser Modifier
@@ -563,17 +556,17 @@ modifier = oneOf [ ( tok TokStar >> return Star )
                  ]
 
 -- just for debugging
-instance Show CP where
-    show (TagName n m) = printableName n++show m
-    show (Choice cps m) = '(': concat (intersperse "|" (map show cps))
-                          ++")"++show m
-    show (Seq cps m) = '(': concat (intersperse "," (map show cps))
-                          ++")"++show m
-instance Show Modifier where
-    show None = ""
-    show Query = "?"
-    show Star = "*"
-    show Plus = "+"
+debugShowCP :: CP -> String
+debugShowCP cp = case cp of
+    TagName n m  -> printableName n++debugShowModifier m
+    Choice cps m -> '(': concat (intersperse "|" (map debugShowCP cps))++")"++debugShowModifier m
+    Seq cps m    -> '(': concat (intersperse "," (map debugShowCP cps))++")"++debugShowModifier m
+debugShowModifier :: Modifier -> String
+debugShowModifier modifier = case modifier of
+    None  -> ""
+    Query -> "?"
+    Star  -> "*"
+    Plus  -> "+"
 ----
 
 mixed :: XParser Mixed
@@ -644,12 +637,12 @@ enumeratedtype =
 notationtype :: XParser NotationType
 notationtype = do
     word "NOTATION"
-    bracket (tok TokBraOpen) (blank (tok TokBraClose))
+    bracket (tok TokBraOpen) (commit $ blank $ tok TokBraClose)
             (peRef name `sepBy1` peRef (tok TokPipe))
 
 enumeration :: XParser Enumeration
 enumeration =
-    bracket (tok TokBraOpen) (blank (tok TokBraClose))
+    bracket (tok TokBraOpen) (commit $ blank $ tok TokBraClose)
             (peRef nmtoken `sepBy1` blank (peRef (tok TokPipe)))
 
 defaultdecl :: XParser DefaultDecl
@@ -716,7 +709,7 @@ newIgnore =
 -- | Return either a general entity reference, or a character reference.
 reference :: XParser Reference
 reference = do
-    myBracket (tok TokAmp) (tok TokSemi) (freetext >>= val)
+    bracket (tok TokAmp) (tok TokSemi) (freetext >>= val)
   where
     val ('#':'x':i) | all isHexDigit i
                     = return . RefChar . fst . head . readHex $ i
@@ -731,11 +724,11 @@ reference =
 
 entityref :: XParser EntityRef
 entityref = do
-    bracket (tok TokAmp) (tok TokSemi) name
+    bracket (tok TokAmp) (commit $ tok TokSemi) name
 
 charref :: XParser CharRef
 charref = do
-    bracket (tok TokAmp) (tok TokSemi) (freetext >>= readCharVal)
+    bracket (tok TokAmp) (commit $ tok TokSemi) (freetext >>= readCharVal)
   where
     readCharVal ('#':'x':i) = return . fst . head . readHex $ i
     readCharVal ('#':i)     = return . fst . head . readDec $ i
@@ -744,7 +737,7 @@ charref = do
 
 pereference :: XParser PEReference
 pereference = do
-    myBracket (tok TokPercent) (tok TokSemi) nmtoken
+    bracket (tok TokPercent) (tok TokSemi) nmtoken
 
 entitydecl :: XParser EntityDecl
 entitydecl =
@@ -829,7 +822,7 @@ encodingdecl :: XParser EncodingDecl
 encodingdecl = do
     (word "encoding" `onFail` word "ENCODING")
     tok TokEqual `onFail` failBadP "expected = in 'encoding' decl"
-    f <- bracket (tok TokQuote) (tok TokQuote) freetext
+    f <- bracket (tok TokQuote) (commit $ tok TokQuote) freetext
     return (EncodingDecl f)
 
 notationdecl :: XParser NotationDecl
@@ -849,7 +842,7 @@ publicid = do
 
 entityvalue :: XParser EntityValue
 entityvalue = do
- -- evs <- bracket (tok TokQuote) (tok TokQuote) (many (peRef ev))
+ -- evs <- bracket (tok TokQuote) (commit $ tok TokQuote) (many (peRef ev))
     tok TokQuote
     pn <- posn
     evs <- many ev
@@ -874,18 +867,18 @@ ev =
 
 attvalue :: XParser AttValue
 attvalue = do
-    avs <- bracket (tok TokQuote) (tok TokQuote)
+    avs <- bracket (tok TokQuote) (commit $ tok TokQuote)
                    (many (either freetext reference))
     return (AttValue avs)
 
 systemliteral :: XParser SystemLiteral
 systemliteral = do
-    s <- bracket (tok TokQuote) (tok TokQuote) freetext
+    s <- bracket (tok TokQuote) (commit $ tok TokQuote) freetext
     return (SystemLiteral s)            -- note: refs &...; not permitted
 
 pubidliteral :: XParser PubidLiteral
 pubidliteral = do
-    s <- bracket (tok TokQuote) (tok TokQuote) freetext
+    s <- bracket (tok TokQuote) (commit $ tok TokQuote) freetext
     return (PubidLiteral s)             -- note: freetext is too liberal here
 
 -- | Return parsed freetext (i.e. until the next markup)
